from pathlib import Path

import rapidjson
import simplejson
from sl10n import SL10n
from sl10n.pimpl import JSONImpl, ORJSONImpl
import ujson

from . import *


def test_parser_impl_simplejson():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path, parsing_impl=JSONImpl(simplejson)).init()

    is_equal(l10n.default_lang, EN)

    locale: Locale = l10n.locale()
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)
    is_equal(locale.topic_title, "Basic 'for' loop algorithm")
    is_equal(locale.topic_text, TOPIC_TEXT_EN)
    is_equal(locale.topic_conclusion, "Now you know basic 'for' loop algorithm!")


def test_parser_impl_rapidjson():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path, parsing_impl=JSONImpl(rapidjson)).init()

    is_equal(l10n.default_lang, EN)

    locale: Locale = l10n.locale()
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)
    is_equal(locale.topic_title, "Basic 'for' loop algorithm")
    is_equal(locale.topic_text, TOPIC_TEXT_EN)
    is_equal(locale.topic_conclusion, "Now you know basic 'for' loop algorithm!")


def test_parser_impl_ujson():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path, parsing_impl=JSONImpl(ujson)).init()

    is_equal(l10n.default_lang, EN)

    locale: Locale = l10n.locale()
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)
    is_equal(locale.topic_title, "Basic 'for' loop algorithm")
    is_equal(locale.topic_text, TOPIC_TEXT_EN)
    is_equal(locale.topic_conclusion, "Now you know basic 'for' loop algorithm!")


def test_parser_impl_orjson():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path, parsing_impl=ORJSONImpl()).init()

    is_equal(l10n.default_lang, EN)

    locale: Locale = l10n.locale()
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)
    is_equal(locale.topic_title, "Basic 'for' loop algorithm")
    is_equal(locale.topic_text, TOPIC_TEXT_EN)
    is_equal(locale.topic_conclusion, "Now you know basic 'for' loop algorithm!")


from __future__ import annotations

from dataclasses import fields
from pathlib import Path
from typing import Any, Type, TypeVar
import warnings

from . import LOGGER, UTF8
from .pimpl import ParsingImpl
from .locale import SLocale
from .modifiers import PreModifiers, PostModifiers
from .warnings import UndefinedLocaleKey, UnexpectedLocaleKey

T = TypeVar('T')


class _LocaleProcess:
    """
    Class that processes locales. Calling this returns a locale container.

    This class is not intended to be inherited.
    """

    EXCLUDE_SIGNAL = 0x01

    filepath: Path
    parsing_impl: ParsingImpl
    data: dict[str, Any]
    used_modifiers: tuple[str, ...]
    all_fields: tuple[str, ...]
    lc_fields: tuple[str, ...]
    all_dumped_fields: tuple[str, ...]

    def __new__(cls, locale_container: Type[T], filepath: Path, parsing_impl: ParsingImpl) -> T | None:
        cls.filepath = filepath
        cls.parsing_impl = parsing_impl

        with open(filepath, encoding=UTF8) as f:
            cls.data = parsing_impl.load(f)

        premodifiers, postmodifiers = cls.parse_modifiers()
        modifiers = dict(premodifiers._asdict(), **postmodifiers._asdict())
        cls.used_modifiers = tuple('$' + k for k, v in modifiers.items() if v is not None)

        signal = cls.apply_premodifiers(premodifiers)
        if signal == cls.EXCLUDE_SIGNAL:
            return

        cls.all_fields = tuple(k.name for k in fields(locale_container))
        cls.lc_fields = tuple(k.name for k in fields(locale_container) if k not in fields(SLocale))
        unexpected_keys = cls.find_unexpected_keys()

        cls.all_dumped_fields = cls.lc_fields + cls.used_modifiers + unexpected_keys

        if cls.any_undefined_key() or unexpected_keys:
            cls.redump()

        cls.apply_postmodifiers(postmodifiers)

        for key in unexpected_keys + cls.used_modifiers:
            del cls.data[key]

        # Join strings in arrays with '\n'
        for key, val in cls.data.items():
            if isinstance(val, list):
                cls.data[key] = '\n'.join(val)

        return locale_container(**cls.data)

    @classmethod
    def parse_modifiers(cls):
        premod, postmod = {}, {}
        for k, v in cls.data.items():
            if k.startswith('$'):
                k = k[1:]
                if k in PreModifiers._fields:
                    premod[k] = v
                elif k in PostModifiers._fields:
                    postmod[k] = v
        return PreModifiers(**premod), PostModifiers(**postmod)

    @classmethod
    def apply_premodifiers(cls, premodifiers: PreModifiers):
        if premodifiers.exclude:
            LOGGER.info(f'Excluding {cls.filepath.name}...')
            return cls.EXCLUDE_SIGNAL

    @classmethod
    def apply_postmodifiers(cls, postmodifiers: PostModifiers):
        if postmodifiers.redump:
            LOGGER.info(f'Redumping {cls.filepath.name}...')
            cls.redump()
        if postmodifiers.lang_code:
            LOGGER.info(f'Changing lang code of "{cls.filepath.name}" to "{postmodifiers.lang_code}"')
            cls.data['lang_code'] = postmodifiers.lang_code
        else:
            cls.data['lang_code'] = cls.filepath.stem

    @classmethod
    def redump(cls):
        with open(cls.filepath, 'w', encoding=UTF8) as f:
            cls.data = {key: cls.data[key] for key in cls.all_dumped_fields}  # fixing pairs order
            cls.parsing_impl.dump(cls.data, f)

    @classmethod
    def any_undefined_key(cls):
        found_undefined_keys = False
        for key in cls.lc_fields:
            if key not in cls.data.keys():
                warnings.warn(f'Found undefined key "{key}" in "{cls.filepath}"', UndefinedLocaleKey, stacklevel=4)
                cls.data[key] = key
                found_undefined_keys = True

        return found_undefined_keys

    @classmethod
    def find_unexpected_keys(cls):
        unexpected_keys = []

        for key in tuple(cls.data):
            if key not in cls.all_fields and key not in cls.used_modifiers:
                warnings.warn(f'Got unexpected key "{key}" in "{cls.filepath}"', UnexpectedLocaleKey, stacklevel=4)
                unexpected_keys.append(key)

        return tuple(unexpected_keys)



from pathlib import Path

import pytest

from sl10n import SL10n
from sl10n.warnings import DefaultLangFileNotFound, SL10nAlreadyInitialized

from . import *


def test_init():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path)

    is_equal(l10n.initialized, False)

    l10n.init()

    is_equal(l10n.initialized, True)


def test_init_default_missing():
    path = Path(__file__).parent / 'data' / 'test_locale_empty'
    l10n = SL10n(Locale, path)

    is_equal(l10n.initialized, False)

    with pytest.warns(DefaultLangFileNotFound):
        l10n.init()

    is_equal(l10n.initialized, True)


def test_warn_init_twice():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path).init()

    with pytest.warns(SL10nAlreadyInitialized):
        l10n.init()



from __future__ import annotations

from dataclasses import fields
import json
from os import PathLike as _PathLike
from pathlib import Path
from typing import Generic, Iterable, Type, TypeVar
import sys
import warnings

if sys.version_info >= (3, 11):
    from typing import Self

from . import UTF8
from .pimpl import ParsingImpl, JSONImpl
from .process import _LocaleProcess as LocaleProcess
from .locale import SLocale
from .modifiers import PreModifiers, PostModifiers
from .warnings import DefaultLangFileNotFound, LangFileAlreadyExists, SL10nAlreadyInitialized, UndefinedLocale


T = TypeVar('T')
PathLike = TypeVar('PathLike', str, _PathLike)


class SL10n(Generic[T]):
    """
    Static text localization system.

    To use it, first create a locale container (it MUST be a subclass of `sl10n.SLocale`)
    and define some string keys in it:
    ```python
    class MyLocale(sl10n.SLocale):
        my_key_1: str
        my_key_2: str
        ...
    ```

    After that, create an SL10n object and pass in your locale container:
    ```python
    l10n = sl10n.Sl10n(MyLocale)
    ```

    Note that it only creates a reference to your localization system. To load all locale files
    and pack their content into locale containers, call `SL10n.init()` method:
    ```python
    l10n = sl10n.Sl10n(MyLocale)
    ...
    l10n.init()
    ```
    """

    default_path = Path.cwd() / 'lang'
    default_pimpl = JSONImpl(json, indent=2, ensure_ascii=False)

    def __init__(self, locale_container: Type[T], path: Path | PathLike = default_path, *, default_lang: str = 'en',
                 ignore_filenames: Iterable[str] = (), parsing_impl: ParsingImpl = default_pimpl):
        """
        Parameters:
            locale_container (Type[T]):
                Locale container to use.
                It must be an SLocale subclass.
            path (str | os.PathLike | pathlib.Path, optional):
                Path to your translation files directory. Defaults to ``pathlib.Path.cwd() / 'lang'``.
            default_lang (str, optional):
                Default language. Defaults to ``'en'``.
            ignore_filenames (Iterable[str], optional):
                What filenames the parser should ignore. Defaults to ``()``.
            parsing_impl (ParsingImpl, optional):
                What JSON parsing implementation to use. Defaults to ``JSONImpl(json, indent=2, ensure_ascii=False)``.

        Raises:
            TypeError: When locale_container is not an SLocale subclass or is an SLocale itself.
        """

        self._check_locale_container(locale_container)

        self.locale_container = locale_container
        self._lc_fields = tuple(k.name for k in fields(locale_container) if k not in fields(SLocale))

        self.path = Path(path)
        self.default_lang = default_lang
        self.ignore_filenames = ignore_filenames
        self.parsing_impl = parsing_impl

        self.locales: dict[str, T] = {}
        self._initialized = False

    @property
    def initialized(self) -> bool:
        return self._initialized

    @staticmethod
    def _check_locale_container(locale_container) -> None:
        if not issubclass(locale_container, SLocale):
            raise TypeError('Your locale container should inherit from Locale class.')
        if locale_container == SLocale:
            raise TypeError(f'Can\'t use {SLocale.__name__} class as locale container.')

    @staticmethod
    def print_modifiers_available():
        """Print all PreModifiers and PostModifiers available."""

        print(f'PreModifiers available: {", ".join("$" + mod for mod in PreModifiers._fields)}')
        print(f'PostModifiers available: {", ".join("$" + mod for mod in PostModifiers._fields)}')

    def init(self) -> Self:
        """
        Load all locale files and pack their content into locale containers.

        You must call it to access your localization.

        Example:
            ```python
            l10n = sl10n.Sl10n(MyLocale)
            ...
            l10n.init()
            ```

            It also returns a reference to your SL10n object, so you can use this oneline to init immediately:
            ```python
            l10n = sl10n.Sl10n(MyLocale).init()
            ```
        Warns:
            SL10nAlreadyInitialized: When Sl10n is already initialized.
        """
        if self._initialized:
            warnings.warn(SL10nAlreadyInitialized(), stacklevel=2)
            return

        if not (self.path / f'{self.default_lang}.json').exists():
            warnings.warn(f'Can\'t find "{self.default_lang}.json" in locales, generating a file...',
                          DefaultLangFileNotFound, stacklevel=2)
            self.create_lang_file(self.default_lang)

        for file in self.path.glob('*.json'):
            if file.stem not in self.ignore_filenames:
                if locale := LocaleProcess(self.locale_container, file, self.parsing_impl):
                    self.locales[file.stem] = locale

        self._initialized = True
        return self

    def locale(self, lang: str | None = None) -> T:
        """
        Returns a Locale object, containing all defined string keys translated to the requested language
        (if such translation exists, otherwise returns a default one).

        Example:
            ```python
            l10n = sl10n.Sl10n(MyLocale).init()

            locale: MyLocale = l10n.locale('en')
            print(locale.my_key_1)
            ```

        Parameters:
            lang (str):
                Language you want to get.

        Raises:
            RuntimeError: When SL10n isn't initialized.

        Tip:
            We do recommend to type hint a variable where you would store a locale container.
            Some IDEs (like PyCharm) may fail to highlight unresolved attributes if you don't do so.
            ```python
            locale: MyLocale = l10n.locale('en')
            ```
        """

        if not self._initialized:
            raise RuntimeError('{0} was not initialized. Perhaps you forgot to call {0}.init()?'
                               .format(self.__class__.__name__))

        if lang is None:
            lang = self.default_lang

        if (locale := self.locales.get(lang)) is None:
            warnings.warn(f'Got unexpected lang "{lang}", returned "{self.default_lang}"',
                          UndefinedLocale, stacklevel=2)
            return self.locales[self.default_lang]

        return locale

    def create_lang_file(self, lang: str, override: bool = False):
        """
        Creates a sample lang file in a requested path.
        If you want to override existing file - set ``override`` to True.

        Useful for fast lang file creation.

        Example:
            ```python
            l10n = sl10n.Sl10n(MyLocale).init()
            l10n.create_lang_file('de')
            ```

        Parameters:
            lang (str):
                Language of translations in this file (used as filename).

            override (bool, optional):
                If ``True``, existing file will be overwritten.
                Defaults to ``False``.

        Warns:
            SL10nAlreadyInitialized: If Sl10n is initialized.
            LangFileAlreadyExists: When the file already exists and ``override`` set to ``False``

        Warning:
            Can be called **only before** SL10n initialization.
        """
        if self._initialized:
            warnings.warn(SL10nAlreadyInitialized('"create_lang_file" can be called only before Sl10n initialization.'),
                          stacklevel=2)
            return

        path = Path(self.path) / f'{lang}.json'
        if override is False and path.exists():
            warnings.warn(f'Lang file "{path}" already exists.', LangFileAlreadyExists, stacklevel=2)
            return

        p = Path(path).parent / f'{self.default_lang}.json'
        if p.exists():
            sample = LocaleProcess(self.locale_container, p, self.parsing_impl)
        else:
            sample = self.locale_container.sample()

        sample = sample.to_dict()

        keys_to_remove = set()
        for k, v in sample.items():
            if k not in self._lc_fields:
                keys_to_remove.add(k)
            if v and '\n' in v:
                sample[k] = v.split('\n')

        for k in keys_to_remove:
            del sample[k]

        if not path.parent.exists():
            path.parent.mkdir(parents=True)

        with open(path, 'w', encoding=UTF8) as f:
            self.parsing_impl.dump(sample, f)



from pathlib import Path

import pytest

from sl10n import SL10n
from sl10n.warnings import DefaultLangFileNotFound, UndefinedLocale, UnexpectedLocaleKey

from . import *


@pytest.mark.parametrize("lang_code", [EN, FR])
def test_locale_empty(lang_code):
    path = Path(__file__).parent / 'data' / 'test_locale_empty'
    with pytest.warns(DefaultLangFileNotFound):
        l10n = SL10n(Locale, path, default_lang=lang_code).init()

    is_equal(l10n.default_lang, lang_code)

    locale = l10n.locale()
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, lang_code)
    is_equal(locale.topic_title, 'topic_title')
    is_equal(locale.topic_text, 'topic_text')
    is_equal(locale.topic_conclusion, 'topic_conclusion')


@pytest.mark.parametrize("lang_code,topic_title,topic_text,topic_conclusion", [
    (EN, "Basic 'for' loop algorithm",
     TOPIC_TEXT_EN, "Now you know basic 'for' loop algorithm!"),
    (FR, "Algorithme de base de la boucle 'for'",
     TOPIC_TEXT_FR, "Vous connaissez maintenant l'algorithme de base de la boucle 'for' !")
])
def test_locale_default(lang_code, topic_title, topic_text, topic_conclusion):
    path = Path(__file__).parent / 'data' / f'test_locale_{lang_code}'
    l10n = SL10n(Locale, path, default_lang=lang_code).init()

    is_equal(l10n.default_lang, lang_code)

    locale = l10n.locale()
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, lang_code)
    is_equal(locale.topic_title, topic_title)
    is_equal(locale.topic_text, topic_text)
    is_equal(locale.topic_conclusion, topic_conclusion)


def test_locale_fr_not_found():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path).init()

    is_equal(l10n.default_lang, EN)

    with pytest.warns(UndefinedLocale):
        locale: Locale = l10n.locale(FR)
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)

    # Must be a default locale (EN in this example)
    is_equal(locale.topic_title, "Basic 'for' loop algorithm")
    is_equal(locale.topic_text, TOPIC_TEXT_EN)
    is_equal(locale.topic_conclusion, "Now you know basic 'for' loop algorithm!")


def test_locale_dynamic_access():
    path = Path(__file__).parent / 'data' / 'test_locale_en'
    l10n = SL10n(Locale, path).init()

    is_equal(l10n.default_lang, EN)

    locale = l10n.locale()

    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)
    is_equal(locale.get('topic_title'), "Basic 'for' loop algorithm")
    is_equal(locale.get('topic_text'), TOPIC_TEXT_EN)
    is_equal(locale.get('topic_conclusion'), "Now you know basic 'for' loop algorithm!")
    with pytest.warns(UnexpectedLocaleKey):
        is_equal(locale.get('unknown_key'), 'unknown_key')



from __future__ import annotations


class DefaultLangFileNotFound(UserWarning):
    """Propogates when the default language file not found."""
    pass


class LangFileAlreadyExists(UserWarning):
    """Propogates when the language file already exists."""
    pass


class SL10nAlreadyInitialized(UserWarning):
    """Propogates when SL10n already initialized."""

    DEFAULT_MESSAGE = 'SL10n already initialized.'

    def __init__(self, message: str | None = None):
        self.message = message if message else self.DEFAULT_MESSAGE

    def __str__(self):
        return self.message


class UndefinedLocaleKey(UserWarning):
    """Propogates when found undefined locale key."""
    pass


class UnexpectedLocaleKey(UserWarning):
    """Propogates when found unexpected locale key."""
    pass


class UndefinedLocale(UserWarning):
    """Propogates when got undefined locale."""
    pass



from __future__ import annotations

from dataclasses import asdict, dataclass, fields
from typing import TypeVar
import warnings

from .warnings import UnexpectedLocaleKey


T = TypeVar('T', bound='SLocale')


DATACLASS_PARAMS = dict(frozen=True)


@dataclass(**DATACLASS_PARAMS)
class SLocale:
    """
    This class contains some specific fields and methods to your locale containers.

    Also, you must subclass your locale container from this class in order to use in ``SL10n``.
    """

    lang_code: str
    """Current locale lang code (filename). Cannot be overwritten even from the file."""

    def __init_subclass__(cls, *args, **kwargs):
        # noinspection PyArgumentList
        return dataclass(cls, **DATACLASS_PARAMS)

    @classmethod
    def sample(cls) -> T:
        """
        Returns:
            A sample locale container with key names as values.

        ``SLocale.lang_code`` sets to None.

        Example:
            ```python
            class MyLocale(sl10n.SLocale):
                my_key_1: str
                my_key_2: str
                ...

            sample = MyLocale.sample()  # MyLocale(lang_code=None, my_key_1='my_key_1', my_key_2='my_key_2', ...)
            ```
        """

        _fields = (k.name for k in fields(cls))
        data = {field: field for field in _fields}
        data['lang_code'] = None
        return cls(**data)

    def to_dict(self) -> dict[str, str]:
        """
        Returns:
            A dict converted from a locale container.

        Example:
            ```python
            class MyLocale(sl10n.SLocale):
                my_key_1: str
                my_key_2: str
                ...

            ...

            locale = l10n.locale('en')
            locale_dict = locale.to_dict()  # {'lang_code': 'en', my_key_1: 'Text 1', my_key_2: 'Text 2', ...}
            ```
        """

        return asdict(self)

    def get(self, key: str) -> str:
        """
        Returns a string associated with the given key (if such
        key exists, otherwise returns the key itself).

        Can be used if the key is known only at runtime.

        Parameters:
            key (str):
                Key used to get string.

        Returns:
            If such key exists, string associated with the given key. Otherwise the key itself.

        Warns:
            UnexpectedLocaleKey: When got an unexpected key.

        Example:
            ```python
            class MyLocale(sl10n.SLocale):
                my_key_1: str
                my_key_2: str
                ...

            ...

            locale = l10n.locale('en')
            my_text_1 = locale.get('my_key_1')  # 'Text 1'
            ```
        """

        try:
            return getattr(self, key)
        except AttributeError:
            warnings.warn(f'Got unexpected key "{key}", returned the key', UnexpectedLocaleKey, stacklevel=2)
            return key



from __future__ import annotations

import json
from types import ModuleType
from typing import Any, IO, NoReturn

from .base import ParsingImpl


__all__ = ('JSONImpl', 'ORJSONImpl')


class JSONImpl(ParsingImpl):
    """
    Interface for basic JSON parsers implementations (which follow builtin `json` interface).

    Supported modules: ``json``, ``simplejson``, ``ujson``, ``rapidjson``
    """
    def __init__(self, module: ModuleType = json, *args, **kwargs):
        self.module = module
        self.args = args
        self.kwargs = kwargs

    def load(self, file: IO):
        return self.module.load(file)

    def dump(self, data: Any, file: IO) -> NoReturn:
        self.module.dump(data, file, *self.args, **self.kwargs)


class ORJSONImpl(JSONImpl):
    """
    Interface for ``orjson`` module.

    Please ensure that you have this module installed.
    """
    def __init__(self, *args, **kwargs):
        import orjson

        super().__init__(orjson, *args, **kwargs)

    def load(self, file: IO, *args, **kwargs):
        return self.module.loads(file.read())

    def dump(self, data: Any, file: IO) -> NoReturn:
        data = self.module.dumps(data, file, *self.args, **self.kwargs)
        file.write(data)



"""Static localization system that reduces the headache of working with localization"""

__version__ = '0.2.0.0'

import logging

UTF8 = 'utf-8'
LOGGER = logging.getLogger('sl10n')

LOGGER.setLevel(logging.INFO)
stream_handler = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s | sl10n: %(message)s', '%H:%M:%S — %d/%m/%Y')
stream_handler.setFormatter(formatter)
LOGGER.addHandler(stream_handler)

from .core import SL10n
from .locale import SLocale



from __future__ import annotations
from typing import NamedTuple


class PreModifiers(NamedTuple):
    """
    Class that contains possible modifiers that activate BEFORE locale processing.

    This class is not intended to be inherited.

    Available modifiers:

    - exclude (``bool``):
      Exclude file from parsing.
    """

    exclude: bool | None = None


class PostModifiers(NamedTuple):
    """
    Class that contains possible modifiers that activate AT or AFTER locale processing.

    This class is not intended to be inherited.

    Available modifiers:

    - redump (``bool``):
      Redump file anyway.
    - lang_code (``str``):
      Override language code of translation file (it doesn't override the access key in ``SL10n.locale()``!).
    """

    redump: bool | None = None
    lang_code: str | None = None



from pathlib import Path

import pytest
from sl10n import SL10n
from sl10n.warnings import SL10nAlreadyInitialized, UndefinedLocale

from . import *


def test_create_file():
    path = Path(__file__).parent / 'data' / 'test_create_file'
    l10n = SL10n(Locale, path)

    l10n.create_lang_file('fr')

    l10n.init()

    locale = l10n.locale(FR)
    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, FR)

    # Must be a copy of default lang file (EN in this example)
    is_equal(locale.topic_title, "Basic 'for' loop algorithm")
    is_equal(locale.topic_text, TOPIC_TEXT_EN)
    is_equal(locale.topic_conclusion, "Now you know basic 'for' loop algorithm!")


def test_create_file_after_init():
    path = Path(__file__).parent / 'data' / 'test_create_file_after_init'
    l10n = SL10n(Locale, path).init()

    with pytest.warns(SL10nAlreadyInitialized):
        l10n.create_lang_file('fr')

    with pytest.warns(UndefinedLocale):
        locale = l10n.locale(FR)

    is_equal(type(locale), Locale)

    is_equal(locale.lang_code, EN)



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any, IO, NoReturn


class ParsingImpl(ABC):
    @abstractmethod
    def load(self, file: IO) -> Any:
        return NotImplemented

    @abstractmethod
    def dump(self, data: Any, file: IO) -> NoReturn:
        return NotImplemented
